/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.cache.pojo.notification.annotation;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import org.jboss.cache.pojo.PojoCache;
import org.jboss.cache.pojo.notification.event.SetModifiedEvent;

// $Id: SetModified.java 6110 2008-06-27 22:06:51Z jason.greene@jboss.com $

/**
 * <p>
 * Indicates that the marked method should be called when an attached set is
 * modified.
 *
 * <p>
 * The method can have any name, but should take a single parameter which is
 * either a {@link SetModifiedEvent} or any superclass. Otherwise, an
 * {@link IllegalArgumentException} will be thrown by
 * {@link PojoCache#addListener(java.lang.Object)}
 *
 * <p>
 * Example:
 *
 * <pre>
 *    &#064PojoCacheSetener
 *    public class MySetener()
 *    {
 *       &#064SetModified
 *       public void handleModified(SetModifiedEvent event)
 *       {
 *          System.out.println(&quot;Set = &quot; + even.getSource());
 *       }
 *    }
 * </pre>
 *
 * @author Jason T. Greene
 */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.METHOD)
public @interface SetModified
{
}
